package de.mlessmann.certassist.repositories;

import static org.assertj.core.api.Assertions.assertThat;

import de.mlessmann.certassist.models.Certificate;
import de.mlessmann.certassist.models.CertificateExtension;
import de.mlessmann.certassist.models.CertificateType;
import jakarta.transaction.Transactional;
import java.util.List;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

@SpringBootTest
class CertificateRepositoryTest {

    @Autowired
    private CertificateRepository repository;

    @Autowired
    private CertificateExtensionRepository extensionRepository;

    private Certificate getCertificate() {
        final Certificate certificate = new Certificate();
        certificate.setOid("test");
        certificate.setCommonName("test-cn");
        certificate.setType(CertificateType.NORMAL_CERTIFICATE);
        certificate.setRequestedKeyLength(1);
        certificate.setRequestedValidityDays(1);

        return certificate;
    }

    @Test
    @Transactional
    void findCertificate() {
        final Certificate certificate = getCertificate();
        repository.save(certificate);

        assertThat(repository.findById(certificate.getId()).orElseThrow().getOid()).isEqualTo("test");
    }

    @Test
    @Transactional
    void findCertificateByOid() {
        final Certificate certificate = getCertificate();
        repository.save(certificate);

        assertThat(repository.findByOid("test").getCommonName()).isEqualTo("test-cn");
    }

    @Test
    @Transactional
    void findCertificateWithExtension() {
        final Certificate certificate = getCertificate();
        CertificateExtension extension = new CertificateExtension();
        extension.setValue("test-ext-value");
        certificate.setCertificateExtension(List.of(extension));
        repository.save(certificate);

        assertThat(repository.findById(certificate.getId()).orElseThrow().getCertificateExtension()).hasSize(1);
        assertThat(extensionRepository.findAll())
            .singleElement()
            .satisfies(ce -> assertThat(ce.getValue()).isEqualTo("test-ext-value"));
    }
}
